@@ -1,8 +1,8 @@
 /*
  * OPENSSL crypto backend implementation
  *
- * Copyright (C) 2010-2016, Red Hat, Inc. All rights reserved.
- * Copyright (C) 2010-2016, Milan Broz
+ * Copyright (C) 2010-2018, Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2010-2018, Milan Broz
  *
  * This file is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -49,31 +49,20 @@ struct crypt_hmac {
 	int hash_len;
 };
 
-int crypt_backend_init(struct crypt_device *ctx)
+/*
+ * Compatible wrappers for OpenSSL < 1.1.0
+ */
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+static void openssl_backend_init(void)
 {
-	if (crypto_backend_initialised)
-		return 0;
-
 	OpenSSL_add_all_algorithms();
-
-	crypto_backend_initialised = 1;
-	return 0;
 }
 
-uint32_t crypt_backend_flags(void)
-{
-	return 0;
-}
-
-const char *crypt_backend_version(void)
+static const char *openssl_backend_version(void)
 {
 	return SSLeay_version(SSLEAY_VERSION);
 }
 
-/*
- * Compatible wrappers for OpenSSL < 1.1.0
- */
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
 static EVP_MD_CTX *EVP_MD_CTX_new(void)
 {
 	EVP_MD_CTX *md = malloc(sizeof(*md));
@@ -105,8 +94,43 @@ static void HMAC_CTX_free(HMAC_CTX *md)
 	HMAC_CTX_cleanup(md);
 	free(md);
 }
+#else
+static void openssl_backend_init(void)
+{
+}
+
+static const char *openssl_backend_version(void)
+{
+    return OpenSSL_version(OPENSSL_VERSION);
+}
 #endif
 
+int crypt_backend_init(struct crypt_device *ctx)
+{
+	if (crypto_backend_initialised)
+		return 0;
+
+	openssl_backend_init();
+
+	crypto_backend_initialised = 1;
+	return 0;
+}
+
+void crypt_backend_destroy(void)
+{
+	crypto_backend_initialised = 0;
+}
+
+uint32_t crypt_backend_flags(void)
+{
+	return 0;
+}
+
+const char *crypt_backend_version(void)
+{
+	return openssl_backend_version();
+}
+
 /* HASH */
 int crypt_hash_size(const char *name)
 {
@@ -275,9 +299,6 @@ int crypt_hmac_destroy(struct crypt_hmac *ctx)
 /* RNG */
 int crypt_backend_rng(char *buffer, size_t length, int quality, int fips)
 {
-	if (fips)
-		return -EINVAL;
-
 	if (RAND_bytes((unsigned char *)buffer, length) != 1)
 		return -EINVAL;
 
@@ -289,21 +310,28 @@ int crypt_pbkdf(const char *kdf, const char *hash,
 		const char *password, size_t password_length,
 		const char *salt, size_t salt_length,
 		char *key, size_t key_length,
-		unsigned int iterations)
+		uint32_t iterations, uint32_t memory, uint32_t parallel)
+
 {
 	const EVP_MD *hash_id;
 
-	if (!kdf || strncmp(kdf, "pbkdf2", 6))
+	if (!kdf)
 		return -EINVAL;
 
-	hash_id = EVP_get_digestbyname(hash);
-	if (!hash_id)
-		return -EINVAL;
+	if (!strcmp(kdf, "pbkdf2")) {
+		hash_id = EVP_get_digestbyname(hash);
+		if (!hash_id)
+			return -EINVAL;
 
-	if (!PKCS5_PBKDF2_HMAC(password, (int)password_length,
-	    (const unsigned char *)salt, (int)salt_length,
-            (int)iterations, hash_id, (int)key_length, (unsigned char *)key))
-		return -EINVAL;
+		if (!PKCS5_PBKDF2_HMAC(password, (int)password_length,
+		    (unsigned char *)salt, (int)salt_length,
+	            (int)iterations, hash_id, (int)key_length, (unsigned char *)key))
+			return -EINVAL;
+		return 0;
+	} else if (!strncmp(kdf, "argon2", 6)) {
+		return argon2(kdf, password, password_length, salt, salt_length,
+			      key, key_length, iterations, memory, parallel);
+	}
 
-	return 0;
+	return -EINVAL;
 }
